""" One of the helpers for the gui application.

    Similar modules: class:`.DataProvider`, :class:`.ParameterSaver`,
    :class:`.NativeArgsSaver`, :class:`.UiLoader`, :class:`.ConfigurationProvider`
"""
from PyQt5.QtCore import QRunnable, QObject, pyqtSignal, pyqtSlot


class WorkerSignals(QObject):
    """ Signals that the worker uses.

        :attr:`.finished`: Signal used to inform about normal finish of the worker job.
        :attr:`.error`: Signal used to inform about finish with error of the worker job.
                        The object parameter is the Error instance.
        :attr:`.result`: Signal used to inform about the result of the worker job.
                         The object parameter is the result of the job.
    """
    finished = pyqtSignal()
    error = pyqtSignal(object)
    result = pyqtSignal(object)


class Worker(QRunnable):
    """ Runs the task in thread and signals about its finish.
    """

    def __init__(self, fn, *args, **kwargs):
        super(Worker, self).__init__()

        self.fn = fn
        self.args = args
        self.kwargs = kwargs
        self.signals = WorkerSignals()

    @pyqtSlot()
    def run(self):
        """ Runs the task and signals about thrown error or given result
        by emiting the WorkerSignals.
        """
        try:
            result = self.fn(*self.args, **self.kwargs)
        except Exception as e:
            self.signals.error.emit(e)
        else:
            self.signals.result.emit(result)
        finally:
            self.signals.finished.emit()



""" One of the helpers for the gui application.

    Similar modules: class:`.NativeArgsSaver`, :class:`.ParameterSaver`,
    :class:`.UiLoader`, :class:`.Worker`, :class:`.DataProvider`
"""
from pathlib import Path
import os.path
from idact.detail.environment.environment_impl import EnvironmentImpl
from idact.detail.environment.environment_text_serialization import \
    serialize_environment


class ConfigurationProvider:
    """ Provides the configuration for idact if it doesn't exist.
    """

    def __init__(self):
        self.filename = ".idact.conf"
        self.home_path = str(Path.home())
        self.conf_file_path = os.path.join(self.home_path, self.filename)

    def create_conf_file(self):
        """ Creates the config file.
        """
        environment_impl = EnvironmentImpl()
        with open(self.conf_file_path, 'w') as file:
            file.write(serialize_environment(environment_impl))

    def check_if_conf_file_exists(self):
        return os.path.exists(self.conf_file_path)



""" One of the helpers for the gui application.

    Similar modules: class:`.NativeArgsSaver`, :class:`.ParameterSaver`,
    :class:`.UiLoader`, :class:`.Worker`, :class:`.ConfigurationProvider`
"""
from PyQt5.QtCore import pyqtSignal, QObject
from idact.core.environment import load_environment
from idact.detail.environment.environment_provider import EnvironmentProvider


class DataProvider(QObject):
    """ Provides the data about clusters inside the .idact.conf file.

        :attr:`.add_cluster_signal`: Signal used to inform about adding a new cluster.
        :attr:`.remove_cluster_signal`: Signal used to inform about removing a cluster.
    """
    add_cluster_signal = pyqtSignal()
    remove_cluster_signal = pyqtSignal()

    def __init__(self):
        super().__init__()
        self.cluster_names = []
        self.load_cluster_names()

    def load_cluster_names(self):
        """ Fetches the cluster names.
        """
        load_environment()
        self.cluster_names = list(EnvironmentProvider().environment.clusters.keys())

    def get_cluster_names(self):
        """ Returns fetched cluster names.
        """
        self.load_cluster_names()
        return self.cluster_names



import sys
from PyQt5.QtWidgets import QApplication

from gui.helpers.configuration_provider import ConfigurationProvider
from gui.functionality.main_window import MainWindow


def main():
    app = QApplication([])
    conf_provider = ConfigurationProvider()
    if not conf_provider.check_if_conf_file_exists():
        conf_provider.create_conf_file()
    main_window = MainWindow()
    main_window.show()
    sys.exit(app.exec_())


if __name__ == '__main__':
    main()



""" One of the widgets that main window composes of.

    See :class:`.MainWindow`
    Similar modules: class:`.AddCluster`, :class:`.IdactNotebook`,
    :class:`.AdjustTimeouts`, :class:`.ManageJobs`
"""
from PyQt5.QtWidgets import QWidget

from idact.core.remove_cluster import remove_cluster
from idact import save_environment, load_environment

from gui.functionality.popup_window import WindowType, PopUpWindow
from gui.helpers.ui_loader import UiLoader
from gui.helpers.worker import Worker
from gui.helpers.custom_exceptions import NoClustersError


class RemoveCluster(QWidget):
    """ Module of GUI that is responsible for the removal of the
    selected cluster.
    """

    def __init__(self, data_provider, parent=None):
        super().__init__(parent=parent)
        self.ui = UiLoader.load_ui_from_file('remove-cluster.ui', self)

        self.parent = parent
        self.data_provider = data_provider

        self.popup_window = PopUpWindow()

        self.ui.remove_cluster_button.clicked.connect(self.concurrent_remove_cluster)

        self.cluster_names = self.data_provider.get_cluster_names()

        self.data_provider.remove_cluster_signal.connect(self.handle_cluster_list_modification)
        self.data_provider.add_cluster_signal.connect(self.handle_cluster_list_modification)
        self.ui.cluster_names_box.addItems(self.cluster_names)

    def concurrent_remove_cluster(self):
        """ Setups the worker that allows to run the remove_cluster functionality
        in the parallel thread.
        """
        worker = Worker(self.remove_cluster)
        worker.signals.result.connect(self.handle_complete_remove_cluster)
        worker.signals.error.connect(self.handle_error_remove_cluster)
        self.parent.threadpool.start(worker)

    def handle_complete_remove_cluster(self):
        """ Handles the completion of removing cluster.
        """
        self.data_provider.remove_cluster_signal.emit()
        self.popup_window.show_message("The cluster has been successfully removed", WindowType.success)

    def handle_error_remove_cluster(self, exception):
        """ Handles the error thrown while removing a cluster.

            :param exception: Instance of the exception.
        """
        if isinstance(exception, NoClustersError):
            self.popup_window.show_message("There are no added clusters", WindowType.error)
        elif isinstance(exception, KeyError):
            self.popup_window.show_message("The cluster does not exist", WindowType.error)
        else:
            self.popup_window.show_message("An error occurred while removing cluster", WindowType.error, exception)

    def remove_cluster(self):
        """ Main function responsible for removing a cluster.
        """
        load_environment()

        cluster_name = str(self.ui.cluster_names_box.currentText())

        if not cluster_name:
            raise NoClustersError()

        remove_cluster(cluster_name)
        save_environment()
        return

    def handle_cluster_list_modification(self):
        """ Handles the modification of the clusters list.
        """
        self.cluster_names = self.data_provider.get_cluster_names()
        self.ui.cluster_names_box.clear()
        self.ui.cluster_names_box.addItems(self.cluster_names)



""" One of the helper widgets.

    See: :class:`.PopUpWindow`, :class:`.ShowLogsWindow`,
    :class:`.YesOrNoWindow`, :class:`.ProgramInfoWindow`,
    :class:`.EditNativeArgumentsWindow`, :class:`.ShowJobsWindow`
"""
from PyQt5.QtWidgets import QWidget
from gui.helpers.ui_loader import UiLoader


class HelpWindow(QWidget):
    """ Shows the help info.
    """
    def __init__(self):
        super().__init__()
        self.ui = UiLoader.load_ui_from_file('help.ui', self)



""" One of the helper widgets.

    See: :class:`.ProgramInfoWindow`, :class:`.PopUpWindow`,
    :class:`.ShowLogsWindow`, :class:`.ShowJobsWindow`,
    :class:`.EditNativeArgumentsWindow`, :class:`.HelpWindow`
"""
from PyQt5.QtWidgets import QWidget, QMessageBox


class YesOrNoWindow(QWidget):
    """ Helper widget window for confirmation.
    """

    def __init__(self):
        super().__init__()
        self.box = QMessageBox(self)
        self.box.addButton(QMessageBox.Yes)
        self.box.addButton(QMessageBox.No)
        self.box.setDefaultButton(QMessageBox.No)
        self.box.setWindowTitle("Warning")
        self.box.setIcon(QMessageBox.Question)

    def show_message(self, message):
        """ Shows the message in the confirmation dialog.

            :param message: Message text to be shown.
        """
        self.box.setText(message)
        ret = self.box.exec_()
        if ret == QMessageBox.Yes:
            return True
        else:
            return False



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="idact-gui",
    version="0.2.3",
    author="Joanna Chyży, Karolina Piekarz, Piotr Swędrak",
    author_email="piotrekswedrak@gmail.com",
    description="A small example package",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/intdata-bsc/idact-gui",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'PyQt5<5.10',
    ],
    dependency_links=[
        'git+ssh://git@github.com/intdata-bsc/idact.git@0.9#egg=idact'
    ],
    entry_points={
        'console_scripts': [
            'idactgui=gui.main:main',
        ],
    }
)



""" One of the helper widgets.

    See: :class:`.ProgramInfoWindow`, :class:`.PopUpWindow`,
    :class:`.YesOrNoWindow`, :class:`.ShowJobsWindow`,
    :class:`.EditNativeArgumentsWindow`, :class:`.HelpWindow`
"""
from PyQt5.QtWidgets import QWidget

from gui.helpers.ui_loader import UiLoader


class ShowLogsWindow(QWidget):
    """ Helper widget window for displaying logs.
    """
    def __init__(self, parent=None):
        super().__init__(parent=parent)
        self.ui = UiLoader.load_ui_from_file('logs.ui', self)

    def write(self, text):
        """ Writes the text to the window.

            :param text: Log text to be shown.
        """
        self.ui.logs_browser.append(text)

    def flush(self):
        pass
